//                                                                  
// Copyright  2013 Lidor Systems.
// All Rights Reserved.
//
// This SOFTWARE is provided "AS IS", WITHOUT WARRANTY OF ANY KIND.
// either express or implied.                        
//

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace Expandable_Groups
{
    public partial class Form1 : Form
    {
        #region Initialization
        public Form1()
        {
            InitializeComponent();

            CreateLabelContent();

            InitList();
        }

        private void CreateLabelContent()
        {
            String content = "<div style=\"font:Verdana,9,r\"><p>   In this sample we will show you how to add expandable groups in ListView control.</p><br></br>";
            content += "<p>   Choose how many groups you want to add, and click on 'Add Groups' button. For this example only 100 groups are allowed. By default each group is expanded showing items that it contains. By clicking on its header or pressing the SPACE key, a group can expand/collapse.</p><br></br>";
            content += "<p>   To make some group always expanded, handle the BeforeGroupCollapse event and prevent the group from collapsing. Check out the source code to see how this is done.</p><br></br>";
            content += "<p><table width=\"100%\" cellpadding=\"5\"><tr><td width=\"100%\" style=\"align:middleright;textcolor:DarkRed;hovertextcolor:Red\"><a href=\"www.lidorsystems.com/support/articles/winforms/listview/\">More Info ...</a></td></tr></table></p>";
            content += "</div>";

            this.labelInfo.Content = content;
        }

        private void labelInfo_ContentObjectClicked(object sender, LidorSystems.IntegralUI.ObjectClickEventArgs e)
        {
            if (e.Object is LinkLabel)
                System.Diagnostics.Process.Start((e.Object as LinkLabel).Links[0].LinkData.ToString());
        }

        private void InitList()
        {
            this.listView1.SuspendUpdate();

            LidorSystems.IntegralUI.Lists.ListViewColumn column = null;
            for (int j = 0; j < 3; j++)
            {
                column = new LidorSystems.IntegralUI.Lists.ListViewColumn("Column " + j.ToString(), "Footer " + j.ToString());
                column.Width = 120;

                this.listView1.Columns.Add(column);
            }

            LidorSystems.IntegralUI.Lists.ListViewItem item = null;
            LidorSystems.IntegralUI.Lists.ListViewSubItem subItem = null;

            for (int i = 0; i < 100; i++)
            {
                item = new LidorSystems.IntegralUI.Lists.ListViewItem("Item " + i.ToString());

                for (int j = 0; j < this.listView1.Columns.Count; j++)
                {
                    subItem = new LidorSystems.IntegralUI.Lists.ListViewSubItem("Item " + i.ToString() + j.ToString());
                    item.SubItems.Add(subItem);
                }

                this.listView1.Items.Add(item);
            }

            this.listView1.ResumeUpdate();
        }
        #endregion

        #region Functionality
        private void btnAddGroups_Click(object sender, EventArgs e)
        {
            // Suspend the update of control layout to increase performance
            this.listView1.SuspendUpdate();

            this.listView1.Groups.Clear();
            comboGroups.Items.Clear();

            LidorSystems.IntegralUI.Lists.ListViewGroup group = null;
            for (int k = 1; k <= numGroups.Value; k++)
            {
                group = new LidorSystems.IntegralUI.Lists.ListViewGroup("Group " + k.ToString());

                this.listView1.Groups.Add(group);
                comboGroups.Items.Add(group.HeaderText);
            }

            // Assign items to groups depending on their index
            int groupIndex = 0;
            for (int i = 0; i < this.listView1.Items.Count; i++)
            {
                groupIndex = i % (int)numGroups.Value;
                this.listView1.Groups[groupIndex].Items.Add(this.listView1.Items[i]);
            }

            // Make sure groups are visible
            this.listView1.ShowGroups = true;

            // Resume the layout and update the control
            this.listView1.ResumeUpdate();

            selectGroup.Enabled = true;
            comboGroups.SelectedText = String.Empty;
        }

        private void listView1_Click(object sender, EventArgs e)
        {
            Point mousePos = this.listView1.ContentPanel.PointToClient(Control.MousePosition);
            LidorSystems.IntegralUI.Lists.ListViewGroup group = this.listView1.GetGroupAt(mousePos.X, mousePos.Y);
            if (group != null)
            {
                Rectangle groupTextRect = group.Bounds;
                groupTextRect.Width -= 20;

                // Make sure group also gets expanded or collapsed when a click is made over group title
                if (groupTextRect.Contains(mousePos))
                {
                    if (group.IsExpanded)
                        group.Collapse();
                    else
                        group.Expand();
                }
            }

        }

        private void listView1_BeforeGroupCollapse(object sender, LidorSystems.IntegralUI.ObjectCancelEventArgs e)
        {
            if (e.Object is LidorSystems.IntegralUI.Lists.ListViewGroup)
            {
                LidorSystems.IntegralUI.Lists.ListViewGroup group = (LidorSystems.IntegralUI.Lists.ListViewGroup)e.Object;

                // If group is marked as always expanded, cancel the collapse of it
                if (group.Tag != null && group.Tag.ToString() == "ALWAYS_EXPANDED")
                    e.Cancel = true;
            }
        }

        private void btnApply_Click(object sender, EventArgs e)
        {
            int groupIndex = comboGroups.SelectedIndex;
            if (groupIndex >= 0 && groupIndex < this.listView1.Groups.Count)
            {
                LidorSystems.IntegralUI.Lists.ListViewGroup group = this.listView1.Groups[groupIndex];

                if (checkExpanded.Checked)
                {
                    // Mark group as always expanded
                    group.Tag = "ALWAYS_EXPANDED";
                    // Make sure to expand the group if it is collapsed
                    if (!group.IsExpanded)
                        group.Expand();

                    // Hide the expand box
                    group.StyleFromParent = false;
                    group.HoverStyle.ButtonColor = Color.Transparent;
                    group.NormalStyle.ButtonColor = Color.Transparent;
                }
                else
                {
                    group.Tag = null;
                    group.StyleFromParent = true;
                }

                // Repaint the control
                this.listView1.Invalidate();
            }
        }
        #endregion
    }
}