'                                                                  
' Copyright  2013 Lidor Systems.
' All Rights Reserved.
'
' This SOFTWARE is provided "AS IS", WITHOUT WARRANTY OF ANY KIND.
' either express or implied.                        
'

Public Class Form1

#Region "Initialization"
    Public Sub New()
        InitializeComponent()

        CreateLabelContent()

        InitList()
    End Sub

    Private Sub CreateLabelContent()
        Dim content As [String] = "<div style=""font:Verdana,9,r""><p>   In this sample we will show you how to add expandable groups in ListView control.</p><br></br>"
        content += "<p>   Choose how many groups you want to add, and click on 'Add Groups' button. For this example only 100 groups are allowed. By default each group is expanded showing items that it contains. By clicking on its header or pressing the SPACE key, a group can expand/collapse.</p><br></br>"
        content += "<p>   To make some group always expanded, handle the BeforeGroupCollapse event and prevent the group from collapsing. Check out the source code to see how this is done.</p><br></br>"
        content += "<p><table width=""100%"" cellpadding=""5""><tr><td width=""100%"" style=""align:middleright;textcolor:DarkRed;hovertextcolor:Red""><a href=""www.lidorsystems.com/support/articles/winforms/listview/"">More Info ...</a></td></tr></table></p>"
        content += "</div>"

        Me.labelInfo.Content = content
    End Sub

    Private Sub labelInfo_ContentObjectClicked(ByVal sender As Object, ByVal e As LidorSystems.IntegralUI.ObjectClickEventArgs) Handles labelInfo.ContentObjectClicked
        If TypeOf e.[Object] Is LinkLabel Then
            System.Diagnostics.Process.Start(TryCast(e.[Object], LinkLabel).Links(0).LinkData.ToString())
        End If
    End Sub

    Private Sub InitList()
        Me.listView1.SuspendUpdate()

        Dim column As LidorSystems.IntegralUI.Lists.ListViewColumn = Nothing
        For j As Integer = 0 To 2
            column = New LidorSystems.IntegralUI.Lists.ListViewColumn("Column " & j.ToString(), "Footer " & j.ToString())
            column.Width = 120

            Me.listView1.Columns.Add(column)
        Next

        Dim item As LidorSystems.IntegralUI.Lists.ListViewItem = Nothing
        Dim subItem As LidorSystems.IntegralUI.Lists.ListViewSubItem = Nothing

        For i As Integer = 0 To 99
            item = New LidorSystems.IntegralUI.Lists.ListViewItem("Item " & i.ToString())

            For j As Integer = 0 To Me.listView1.Columns.Count - 1
                subItem = New LidorSystems.IntegralUI.Lists.ListViewSubItem("Item " & i.ToString() & j.ToString())
                item.SubItems.Add(subItem)
            Next

            Me.listView1.Items.Add(item)
        Next

        Me.listView1.ResumeUpdate()
    End Sub
#End Region

#Region "Functionality"
    Private Sub btnAddGroups_Click(ByVal sender As Object, ByVal e As EventArgs) Handles btnAddGroups.Click
        ' Suspend the update of control layout to increase performance
        Me.listView1.SuspendUpdate()

        Me.listView1.Groups.Clear()
        comboGroups.Items.Clear()

        Dim group As LidorSystems.IntegralUI.Lists.ListViewGroup = Nothing
        For k As Integer = 1 To numGroups.Value
            group = New LidorSystems.IntegralUI.Lists.ListViewGroup("Group " & k.ToString())

            Me.listView1.Groups.Add(group)
            comboGroups.Items.Add(group.HeaderText)
        Next

        ' Assign items to groups depending on their index
        Dim groupIndex As Integer = 0
        For i As Integer = 0 To Me.listView1.Items.Count - 1
            groupIndex = i Mod CInt(numGroups.Value)
            Me.listView1.Groups(groupIndex).Items.Add(Me.listView1.Items(i))
        Next

        ' Make sure groups are visible
        Me.listView1.ShowGroups = True

        ' Resume the layout and update the control
        Me.listView1.ResumeUpdate()

        selectGroup.Enabled = True
        comboGroups.SelectedText = [String].Empty
    End Sub

    Private Sub listView1_Click(ByVal sender As Object, ByVal e As EventArgs) Handles listView1.Click
        Dim mousePos As Point = Me.listView1.ContentPanel.PointToClient(Control.MousePosition)
        Dim group As LidorSystems.IntegralUI.Lists.ListViewGroup = Me.listView1.GetGroupAt(mousePos.X, mousePos.Y)
        If group IsNot Nothing Then
            Dim groupTextRect As Rectangle = group.Bounds
            groupTextRect.Width -= 20

            ' Make sure group also gets expanded or collapsed when a click is made over group title
            If groupTextRect.Contains(mousePos) Then
                If group.IsExpanded Then
                    group.Collapse()
                Else
                    group.Expand()
                End If
            End If
        End If

    End Sub

    Private Sub listView1_BeforeGroupCollapse(ByVal sender As Object, ByVal e As LidorSystems.IntegralUI.ObjectCancelEventArgs) Handles listView1.BeforeGroupCollapse
        If TypeOf e.[Object] Is LidorSystems.IntegralUI.Lists.ListViewGroup Then
            Dim group As LidorSystems.IntegralUI.Lists.ListViewGroup = DirectCast(e.[Object], LidorSystems.IntegralUI.Lists.ListViewGroup)

            ' If group is marked as always expanded, cancel the collapse of it
            If group.Tag IsNot Nothing AndAlso group.Tag.ToString() = "ALWAYS_EXPANDED" Then
                e.Cancel = True
            End If
        End If
    End Sub

    Private Sub btnApply_Click(ByVal sender As Object, ByVal e As EventArgs) Handles btnApply.Click
        Dim groupIndex As Integer = comboGroups.SelectedIndex
        If groupIndex >= 0 AndAlso groupIndex < Me.listView1.Groups.Count Then
            Dim group As LidorSystems.IntegralUI.Lists.ListViewGroup = Me.listView1.Groups(groupIndex)

            If checkExpanded.Checked Then
                ' Mark group as always expanded
                group.Tag = "ALWAYS_EXPANDED"
                ' Make sure to expand the group if it is collapsed
                If Not group.IsExpanded Then
                    group.Expand()
                End If

                ' Hide the expand box
                group.StyleFromParent = False
                group.HoverStyle.ButtonColor = Color.Transparent
                group.NormalStyle.ButtonColor = Color.Transparent
            Else
                group.Tag = Nothing
                group.StyleFromParent = True
            End If

            ' Repaint the control
            Me.listView1.Invalidate()
        End If
    End Sub
#End Region

End Class
